/**
 * 
 */
package org.inbio.gwt.associatedto.client.ui;

import org.inbio.gwt.associatedto.client.dto.AssociatedToConstants;
import org.inbio.gwt.associatedto.client.dto.AssociatedToInfo;
import org.inbio.gwt.associatedto.client.listener.AssociatedToListener;

import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FocusListener;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RadioButton;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * @author jgutierrez
 * 
 */
public class AssociatedTo extends Composite implements ClickListener,
		FocusListener {

	private AssociatedToListener listener;

	// interface elements:
	private HorizontalPanel main = new HorizontalPanel();

	private VerticalPanel radioButtonPanel;

	private RadioButton specimenNumberButton;

	private RadioButton ObservationCodeButton;

	private RadioButton gatheringNumberButton;

	private RadioButton noAssociationButton;

	private VerticalPanel textPanel;

	private TextBox textInput;

	private Label infoFedBack;

	private Image validImage = new Image("images/invalid.png");

	// variables
	// defaults -- ..:wired:..
	private Integer actualAssociationType = AssociatedToConstants.NO_ASSOCIATION;

	/**
	 * Widget Constructor
	 * 
	 * @param language
	 * @param associationListener
	 * @param associatedToCode
	 * @param associatedToValue
	 */
	public AssociatedTo(Integer language,
			AssociatedToListener associationListener, Integer associatedToCode,
			String associatedToValue) {

		listener = associationListener;

		radioButtonPanel = new VerticalPanel();

		// specimen number
		specimenNumberButton = new RadioButton("associationRadioGroup",
				"Número de Espécimen");
		specimenNumberButton.addClickListener(this);

		// observation code
		ObservationCodeButton = new RadioButton("associationRadioGroup",
				"Número de Observación");
		ObservationCodeButton.addClickListener(this);

		// collect number
		gatheringNumberButton = new RadioButton("associationRadioGroup",
				"Id. de Colecta");
		gatheringNumberButton.addClickListener(this);

		// no association
		noAssociationButton = new RadioButton("associationRadioGroup",
				"Sin Asociación");
		noAssociationButton.addClickListener(this);

		// rediobutton to the left and textbox to the rigth
		radioButtonPanel.add(specimenNumberButton);
		radioButtonPanel.add(ObservationCodeButton);
		radioButtonPanel.add(gatheringNumberButton);
		radioButtonPanel.add(noAssociationButton);

		// sets the main panel
		main.setSpacing(15);
		main.add(radioButtonPanel);

		textPanel = new VerticalPanel();
		textPanel.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);

		textInput = new TextBox();
		textInput.addFocusListener(this);
		textPanel.add(textInput);

		infoFedBack = new Label();
		textPanel.add(infoFedBack);

		main.add(textPanel);
		validImage.setVisible(false);
		main.add(validImage);

		initWidget(main);

		// sets the default values
		setAssociatedToInfo(new AssociatedToInfo(associatedToCode, associatedToValue));
		isValid();
	}

	/**
	 * Sets the value and type of the association
	 * 
	 * @param ati
	 */
	public void setAssociatedToInfo(AssociatedToInfo ati) {
		specimenNumberButton.setChecked(false);
		ObservationCodeButton.setChecked(false);
		gatheringNumberButton.setChecked(false);
		noAssociationButton.setChecked(false);

		if (ati.getType().equals(AssociatedToConstants.SPECIMEN_NUMBER)) {
			specimenNumberButton.setChecked(true);

		} else if (ati.getType().equals(AssociatedToConstants.OBSERVATION_NUMBER)) {
			ObservationCodeButton.setChecked(true);

		} else if (ati.getType().equals(AssociatedToConstants.GATHERING_CODE)) {
			gatheringNumberButton.setChecked(true);

		} else if (ati.getType().equals(AssociatedToConstants.NO_ASSOCIATION)) {
			noAssociationButton.setChecked(true);
			textInput.setEnabled(false);
		}

		// asociationValues
		textInput.setText(ati.getValue());
	}

	/**
	 * Gets the value and type of the association
	 * 
	 * @return
	 */
	public AssociatedToInfo getAssociatedToInfo(){
		
		return new AssociatedToInfo(this.actualAssociationType, textInput.getText());
	}

	/**
	 * fired when a radio button is clicked. The event is manages in the container
	 * class of this widget
	 * 
	 * If the AssociationValue is null a '-1' is passed as the 'associationValue'
	 * parameter on the onAssociationChange method of the listener
	 * 
	 */
	public void onClick(Widget sender) {

		RadioButton clicked = (RadioButton) sender;

		// if the no association is selected
		if (clicked.equals(noAssociationButton)) {
			actualAssociationType = AssociatedToConstants.NO_ASSOCIATION;
			textInput.setText("");
			textInput.setEnabled(false);

		} else {

			textInput.setEnabled(true);

			if (clicked.equals(specimenNumberButton))
				actualAssociationType = AssociatedToConstants.SPECIMEN_NUMBER;

			else if (clicked.equals(ObservationCodeButton))
				actualAssociationType = AssociatedToConstants.OBSERVATION_NUMBER;

			else if (clicked.equals(gatheringNumberButton))
				actualAssociationType = AssociatedToConstants.GATHERING_CODE;
		}

		if (isValid())
			listener.onAssociationChange(actualAssociationType, textInput.getText());
		// fireEvent();

	}// onClick

	// focuslistener
	public void onFocus(Widget sender) {
		// DO nothing
	}

	// focuslistener
	public void onLostFocus(Widget sender) {
		// fireEvent();
		if (isValid())
			listener.onAssociationChange(actualAssociationType, textInput.getText());
	}

	/**
	 * Checks if the widget is valid. If not shows in the GUI a message and a
	 * image of error.
	 * 
	 * @return
	 */
	private boolean isValid() {

		String associationValue = textInput.getText();
		String feedbackText = "";
		boolean valid = false;
		System.out.println("inicia en isValid");

		// If the Type is no association almost every value is valid
		if (actualAssociationType.equals(AssociatedToConstants.NO_ASSOCIATION)) {
			valid = associationValue.equals("");
		}

		// fos specimen and observation numbers the only invalid clause is that the
		// textualValue must be parseable to numeric Object, in this case Integer.
		else if (actualAssociationType
				.equals(AssociatedToConstants.SPECIMEN_NUMBER)
				|| actualAssociationType
						.equals(AssociatedToConstants.OBSERVATION_NUMBER)) {

			try {
				Integer.parseInt(associationValue);
				valid = true;
			} catch (NumberFormatException nfe) {
				feedbackText = "Falta un valor numérico.";
			}
		}

		// Valid gathering code's must be of the pattern "STRING;NUMBER"
		else if (actualAssociationType.equals(AssociatedToConstants.GATHERING_CODE)) {

			int semicolon = associationValue.lastIndexOf(";");
			if (semicolon == -1
					|| associationValue.substring(0, semicolon).length() == 0) {
				feedbackText = "Utilice ';' para separar el nombre del código.";
			} else {
				try {
					Integer.parseInt(associationValue.substring(semicolon + 1));
					valid = true;
				} catch (NumberFormatException nfe) {
					feedbackText = "Falta un valor numérico despues del ';'.";
				}
			}
		}

		validImage.setVisible(!valid);
		infoFedBack.setText(feedbackText);
		return valid;
	}
}
