/**
 * 
 */
package org.inbio.gwt.controlledtext.client.ui;



import java.util.ArrayList;
import java.util.List;

import org.inbio.gwt.controlledtext.client.dto.TextInfo;
import org.inbio.gwt.controlledtext.client.rpc.ControlledTextRPC;
import org.inbio.gwt.controlledtext.client.rpc.ControlledTextRPCAsync;


import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.MultiWordSuggestOracle;
import com.google.gwt.user.client.ui.SuggestBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * @author jgutierrez
 *
 */
public class TextSelector extends Composite implements ClickListener, KeyboardListener, AsyncCallback {
	
	private ControlledTextRPCAsync rpc;

	// main panel
	private HorizontalPanel main;

	private VerticalPanel left;

	private VerticalPanel center;
	
	private Image loading;
	
	private SuggestBox input;

	//private Button search;

	private Button add;

	private Button del;

	private ListBox visibleSelectedTexts;
	
	
	/**
	 * Class constructor
	 * 
	 */
	public TextSelector(String RPCEntryPoint) {
		//init RPC
		rpc = (ControlledTextRPCAsync) GWT.create(ControlledTextRPC.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) rpc;
		endpoint.setServiceEntryPoint(GWT.getModuleBaseURL() + RPCEntryPoint);
		
		main = new HorizontalPanel();
		main.setSpacing(10);
		main.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
		main.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);

		
		left = new VerticalPanel();
		left.setSpacing(10);
		loading = new Image("images/loading.gif");
		left.add(loading);
		//search = new Button("buscar", this);
		//search.setEnabled(false);
		//left.add(search);
		main.add(left);

		//
		center = new VerticalPanel();
		center.setSpacing(10);
		add = new Button("agregar", this);
		center.add(add);
		del = new Button("eliminar", this);
		center.add(del);
		main.add(center);

		// 
		visibleSelectedTexts = new ListBox();
		visibleSelectedTexts.setVisibleItemCount(4);
		main.add(visibleSelectedTexts);

		
		initWidget(main);
		
		rpc.getAllTexts(this);
			
	}
	
	public void onFailure(Throwable caught) {
		// TODO Auto-generated method stub
	}

	@SuppressWarnings("unchecked")
	public void onSuccess(Object result) {
		MultiWordSuggestOracle suggestions = new MultiWordSuggestOracle();
		suggestions.addAll((List<String>) result);
		input = new SuggestBox(suggestions);
		input.addKeyboardListener(this);
		loading.removeFromParent();
		left.add(input);
	}
	
	
	/**
	 * Click listenter para los botones add, del
	 * 
	 */
	public void onClick(Widget sender) {
		Button clicked = (Button) sender;

		if (clicked.equals(add)) {

			String text = input.getText();
			if (checkInput(text))
				insertText(text);

		} else if (clicked.equals(del)) {

			int selected = visibleSelectedTexts.getSelectedIndex();
			visibleSelectedTexts.removeItem(selected);

		} //else if (clicked.equals(search)) {
		//todo
		//}
	}
	
	/**
	 * Gets the text from the input textbox and verifies it.
	 * 
	 * @param text
	 *          text to be added to the list
	 */
	private void insertText(String text) {

		rpc.getTextInfo(text, new AsyncCallback() {

		 public void onFailure(Throwable caught) {
			 Window.alert("Error: " + caught.getMessage());
			 input.setText("");
		 }

		 public void onSuccess(Object result) {
		 TextInfo ti = (TextInfo) result;

		 if (ti != null) {
			 visibleSelectedTexts.addItem(ti.getName(), String.valueOf(ti.getId()));
			 input.setText("");
		 }

		 }

		 });

	}
	
	/**
	 * 
	 * @param inputText
	 * @return true is the input apears to be ok.
	 */
	private boolean checkInput(String inputText) {
		if (inputText.length() == 0) {
			//Window.alert("Error: debe ingresar algo.");
			return false;
		}
		return true;
	}
	

/**
 * Asociado al SuggestBox. on KeyDown se ignorará
 * 
 * @param sender
 * @param keyCode
 * @param modifiers
 */
	public void onKeyDown(Widget sender, char keyCode, int modifiers) {	}

/**
 * Asociado al SuggestBox. on onKeyPress se ignorará
 * 
 * @param sender
 * @param keyCode
 * @param modifiers
 */
	public void onKeyPress(Widget sender, char keyCode, int modifiers) { }

/**
 * 
 * @param sender
 * @param keyCode
 * @param modifiers
 */
	public void onKeyUp(Widget sender, char keyCode, int modifiers) {
		//si fue el enter el botón presionado se interpretará como si se hubiera presionado 
		//botón de agregar.
		if (keyCode == KEY_ENTER){
			//Window.alert("enter");
			onClick(this.add);
		
		} 
			
	}
	
	/*********************************************************************************/
	/** Gets y Sets principales del widget																					**/
	/*********************************************************************************/
	
	/**
	 * 
	 * @param textInfoList
	 */
	public void setSelectedTexts(List<TextInfo> textInfoList) {
		for(TextInfo ti : textInfoList)
			visibleSelectedTexts.addItem(ti.getName(), String.valueOf(ti.getId()));
	}

	/**
	 * 
	 * @return
	 */
	public List<TextInfo> getSelecetedTexts() {
		List<TextInfo> textInfoList = new ArrayList<TextInfo>();
		TextInfo ti;
		String name;
		String id;
		
		for(int i=0; i<visibleSelectedTexts.getItemCount(); i++){
			name = visibleSelectedTexts.getItemText(i);
			id = visibleSelectedTexts.getValue(i);
			ti = new TextInfo(Integer.parseInt(id), name);
			textInfoList.add(ti);
		}
		
		return textInfoList;
	}
	

	

}
