/**
 * 
 */
package org.inbio.gwt.taxonomy.client.widget;

import java.util.ArrayList;
import java.util.List;

import org.inbio.gwt.taxonomy.client.dto.TaxonInfo;
import org.inbio.gwt.taxonomy.client.events.TaxonListener;
import org.inbio.gwt.taxonomy.client.rpc.TaxonSelectorRPC;
import org.inbio.gwt.taxonomy.client.rpc.TaxonSelectorRPCAsync;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * @author jgutierrez
 * 
 */
public class TaxonSelector extends Composite implements ClickListener,
		TaxonListener {

	private TaxonSelectorRPCAsync rpc;

	// main panel
	private HorizontalPanel main = new HorizontalPanel();

	private VerticalPanel left = new VerticalPanel();

	private VerticalPanel center = new VerticalPanel();

	private TextBox input = new TextBox();

	private Button search = new Button("buscar", this);

	private Button add = new Button("agregar", this);

	private Button del = new Button("eliminar", this);

	private ListBox taxons = new ListBox();

	/**
	 * Class constructor
	 * 
	 */
	public TaxonSelector(String RPCEntryPoint) {
		//init RPC
		rpc = (TaxonSelectorRPCAsync) GWT.create(TaxonSelectorRPC.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) rpc;
		endpoint.setServiceEntryPoint(GWT.getModuleBaseURL() + RPCEntryPoint);

		main.setSpacing(10);
		main.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
		main.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);

		left.setSpacing(10);
		left.add(input);
		search.setEnabled(false);
		left.add(search);
		main.add(left);

		center.setSpacing(10);
		center.add(add);
		center.add(del);
		main.add(center);

		// Make enough room for all five items (setting this value to 1 turns it
		// into a drop-down list).
		taxons.setVisibleItemCount(4);
		// Add it to the root panel.
		main.add(taxons);

		initWidget(main);
	}

	/**
	 * When some button is click
	 */
	public void onClick(Widget sender) {
		Button clicked = (Button) sender;

		if (clicked.equals(search)) {

			Window.alert("to do");

		} else if (clicked.equals(add)) {

			String taxonName = input.getText();

			if (checkInput(taxonName)) {
				preAddTaxon(taxonName);
			}

		} else if (clicked.equals(del)) {

			int selected = taxons.getSelectedIndex();
			taxons.removeItem(selected);

		}

	}

	/**
	 * 
	 * @param inputText
	 * @return true is the input apears to be ok.
	 */
	private boolean checkInput(String inputText) {

		if (inputText.length() == 0) {
			Window.alert("Error: debe ingresar el nombre de un taxón. ");
			return false;
		}

		return true;
	}

	/**
	 * Gets the text from the input textbox, verifies the taxon in order to get
	 * its taxonId from atta, then adds the ID to the list(box)
	 * 
	 * @param taxonName
	 *          taxon to be added to the list
	 */
	private void preAddTaxon(String taxonName) {

		rpc.getTaxonInfo(taxonName, new AsyncCallback() {

		 public void onFailure(Throwable caught) {
		 Window.alert("Error: " + caught.getMessage());

		 }

		 public void onSuccess(Object result) {
		 List taxonsInfo = (List) result;

		 if (taxonsInfo.size() == 1) {
		 addTaxon((TaxonInfo) taxonsInfo.get(0));

		 } else {
		// mas de un taxon....
		 //Window.alert("mas de un taxon... trabajar en eso...");
		 manageMultipleTaxons(taxonsInfo);
		 }

		 }

		 });

	}

	/**
	 * Adds a taxon to the list, doesnt check if the info is ok
	 * 
	 * @param taxon
	 *          a TaxonInfo object
	 */
	private void addTaxon(TaxonInfo taxon) {
		String textToShow = taxon.getTaxonName() + " [reino: "
				+ taxon.getKingdomName() + "]";

		taxons.addItem(textToShow, taxon.getTaxonId());
	}

	/**
	 * Creates a new multipleTaxons widget 
	 * 
	 * @param taxonsList
	 */
	private void manageMultipleTaxons(List taxonsList) {
		MultipleTaxonSelector mts = new MultipleTaxonSelector(taxonsList, this);
		mts.show();
	}

	/**
	 * fired when the multiple taxons selector sends a mesage that the user has
	 * selected a taxon
	 */
	public void onTaxonSelected(String textToShow, String taxonId) {
		taxons.addItem(textToShow, taxonId);

	}


	/**
	 * 
	 * @param taxonsInfo
	 */
	public void setTaxonomy(List<String> literalTaxonIds) {

		rpc.getTaxonsInfo(literalTaxonIds, new AsyncCallback() {

			public void onFailure(Throwable caught) {
				Window.alert("Error: " + caught.getMessage());
			}

			public void onSuccess(Object result) {
				List taxonsInfo = (List) result;

				for (int i = 0; i < taxonsInfo.size(); i++) {
					addTaxon((TaxonInfo) taxonsInfo.get(i));
				}
			}

		});

	}

	/**
	 * 
	 * @return a list with the taxonIds of the elements selected as a String, in
	 *         oder words return a list of string, each one represents the taxonId
	 *         of the taxonomy selected
	 */
	public List<String> getTaxonomy() {
		List<String> taxonomy = new ArrayList<String>();

		for (int i = 0; i < taxons.getItemCount(); i++) {
			taxonomy.add(taxons.getValue(i));
		}

		return taxonomy;
	}

}
